@@ -9,12 +9,13 @@
 #                        |  |_|  /  |  |  /  |  |  / \_
 #                         \/  |_/   |_/|_/\_/|_/|_/ \/
 
-__version__ = '0.1.0'
+__version__ = '0.1.1'
 
 import re
 
-# TODO  multiple Whens for one Scenario!
-# TODO  what happens with blank table items?
+#  TODO  multiple Whens for one Scenario!
+#  TODO  what happens with blank table items?
+#  ERGO  river is to riparian as pond is to ___?
 
 class Morelia:
 
@@ -73,7 +74,6 @@ class Morelia:
     def format_fault(self, diagnostic):
         parent_reconstruction = ''
         if self.parent:  parent_reconstruction = self.parent.reconstruction().replace('\n', '\\n')
-#        if self.my_scene():  parent_reconstruction = self.my_scene().reconstruction().replace('\n', '\\n') + parent_reconstruction
         reconstruction = self.reconstruction().replace('\n', '\\n')
         args = (self.get_filename(), self.line_number, parent_reconstruction, reconstruction, diagnostic)
         return '\n  File "%s", line %s, in %s\n    %s\n%s' % args
@@ -114,8 +114,6 @@ class Viridis(Morelia):
 
         suite.fail(diagnostic)
 
-#  ERGO  river is to riparian as pond is to ___?
-
     def suggest_doc_string(self, predicate = None):  #  CONSIDER  invent Ruby scan here, to dazzle the natives
         self.extra_arguments = ''
         if not predicate:  predicate = self.predicate
@@ -150,7 +148,7 @@ class Viridis(Morelia):
             self.method_name = s
             method = suite.__getattribute__(s)
             doc = method.__doc__
-            
+
             if doc:
                 doc = re.compile('^' + doc + '$')  #  CONSIDER deal with users who put in the ^$
                 m = doc.match(self.augment_predicate())
@@ -268,11 +266,16 @@ class ReportVisitor:
     def __init__(self, suite):  self.suite = suite
     string = ''
 
+    def permute_schedule(self, node):  return [[0]]
+
     def visit(self, node):
-        recon =  node.reconstruction()
+        recon, we_owe =  node.to_html()
         if recon[-1] != '\n':  recon += '\n'  #  TODO  clean this outa def reconstruction(s)!
         self.string += recon
+        return we_owe
 
+    def owed(self, owed):  self.string += owed
+        
     def __str__(self):
         return self.string
 
@@ -280,11 +283,15 @@ class ReportVisitor:
 class TestVisitor:
     def __init__(self, suite):  self.suite = suite
 
+    def permute_schedule(self, node):  return node.permute_schedule()
+        
     def visit(self, node):
         # print node.reconstruction()  # CONSIDER  if verbose
         self.suite.step = node
         node.evaluate_step(self)
 
+    def owed(self, igme):  pass
+
 
 class Feature(Morelia):
     def my_parent_type(self):  return None
@@ -292,12 +299,16 @@ class Feature(Morelia):
     def evaluate_step(self, v):  
         self.enforce(0 < len(self.steps), 'Feature without Scenario(s)')
 
+    def to_html(self):
+        return ['\n<div><table><tr><td align="right" valign="top"><em>' + self.concept + '</em>:</td><td colspan="101">' + 
+                                    _clean_html(self.predicate) + '</td></tr></table></div>', '']
+
 
 class Scenario(Morelia):
     def my_parent_type(self):  return Feature
 
     def evaluate_steps(self, visitor):
-        schedule = self.permute_schedule()
+        schedule = visitor.permute_schedule(self)
 
         for indices in schedule:
             self.row_indices = indices
@@ -312,8 +323,9 @@ class Scenario(Morelia):
         visitor.suite.setUp()
 
         try:
-            visitor.visit(self)
+            u_owe = visitor.visit(self)
             for step in self.steps:  step.evaluate_steps(visitor)
+            visitor.owed(u_owe)
         finally:
             visitor.suite.tearDown()
 
@@ -336,6 +348,10 @@ class Scenario(Morelia):
     def reconstruction(self):
         return '\n' + self.concept + ': ' + self.predicate
 
+    def to_html(self):
+        return '\n<div><table><tr><td align="right" valign="top"><em>' + self.concept + '</em>:</td><td colspan="101">' + \
+                                            _clean_html(self.predicate) + '</td></tr>', '</table></div>'
+
 
 class Step(Viridis):
     def my_parent_type(self):  return Scenario
@@ -393,6 +409,9 @@ class Step(Viridis):
         self.copy = self.copy.replace('<'+self.replitron+'>', found)
 
         # CONSIDER  mix replitrons and matchers!
+        
+    def to_html(self):
+        return '\n<tr><td align="right" valign="top"><em>' + self.concept + '</em></td><td colspan="101">' + _clean_html(self.predicate) + '</td></tr>', ''
 
 
 class Given(Step):   #  CONSIDER  distinguish these by fault signatures!
@@ -406,6 +425,7 @@ class And(Step):
 
 #  CONSIDER  how to validate that every row you think you wrote actually ran?
 
+
 class Row(Morelia):
     def i_look_like(self):  return r'\|'
     def my_parent_type(self):  return Step
@@ -416,6 +436,24 @@ class Row(Morelia):
         if recon[-1] != '\n':  recon += '\n'
         return recon
 
+    def to_html(self):
+        html = '\n<tr><td></td>'
+        idx = self.parent.steps.index(self)
+        em = 'span'
+        if idx == 0:
+            color = 'silver'
+            em = 'em'
+        elif ((2 + idx) / 3) % 2 == 0:  
+            color = '#eeffff'
+        else:
+            color = '#ffffee'
+        
+        for col in self.harvest():
+            html += '<td style="background-color: %s;"><%s>' % (color, em) + _clean_html(col) + '</%s></td>' % em
+            
+        html += '</tr>'
+        return html, ''
+
     def count_dimension(self):
         if self is self.parent.steps[0]:  return 0
         return 1  #  TODO  raise an error (if the table has one row!)
@@ -444,6 +482,9 @@ class Comment(Morelia):
         if recon[-1] != '\n':  recon += '\n'
         return recon
 
+    def to_html(self):
+        return '\n# <em>' + _clean_html(self.predicate) + '</em><br/>', ''
+
 if __name__ == '__main__':
     import os
     os.system('python ../tests/morelia_suite.py')   #  NOTE  this might not return the correct shell value
@@ -477,6 +518,10 @@ def _imap(function, *iterables):
         else:
             yield function(*args)
 
+def _clean_html(string):
+    return string.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;'). \
+                            replace('"', '&quot;').replace("'", '&#39;')
+
 #  CONSIDER  display all missing steps not just the first
 #  ERGO  Morelia should raise a form in any state!
 #  ERGO  get Morelia working with more Pythons - virtualenv it!
